let both    = require('../both');
let sinonChai = require('sinon-chai');
let sinon = require('sinon');
let chai    = require('chai');
chai.use(sinonChai);
let expect  = chai.expect;

describe('Command Module', () => {

  describe('Commands error handling', () => {

    it('throws error if no argument is given', () => {
      expect(() => {
        both.command();
      }).to.throw(Error);
    });

    it('throws error if the commandName isn`t a string', () => {
      expect(() => {
        both.command(123);
      }).to.throw(Error);
    });

    it('throws error if no commandFunction is given or the wrong type was shipped', () => {
      expect(() => {
        both.command('someCommand');
      }).to.throw(Error);
    });

    it('throws error if the given commandFunction is not a function ', () => {
      expect(() => {
        both.command('someCommand', '');
      }).to.throw(Error);

      expect(() => {
        both.command('someCommand', 123);
      }).to.throw(Error);

      expect(() => {
        both.command('someCommand', []);
      }).to.throw(Error);

      expect(() => {
        both.command('someCommand', {});
      }).to.throw(Error);
    });
  });

  describe('defining a command', () => {

    let dummyCommandFunc;

    beforeEach(() => {
      dummyCommandFunc = sinon.spy();
      both._globalRegistery.command = {};
    });

    it('registers the function in the both-registery', () => {

      both.register('command', {
        name: 'dummyCommand',
        func: dummyCommandFunc
      });

      expect(both._globalRegistery.command.dummyCommand).to.deep.equal({
        name: 'dummyCommand',
        func: dummyCommandFunc
      });

    });
  });
});
