let both    = require('../both');
let sinonChai = require('sinon-chai');
let sinon = require('sinon');
let chai    = require('chai');
chai.use(sinonChai);
let expect  = chai.expect;

describe('On Module', () => {
  let dummyCommandFunc;

  beforeEach(() => {
    dummyCommandFunc = sinon.spy();

    both._globalRegistery = {};
  });

  describe('Listeners error handling', () => {

    it('throws error if more than two params are shipped', () => {
      expect(() => {
        both.on('dummyCommand', dummyCommandFunc, 'thirdNotAllowedParam');
      }).to.throw(Error);
    });

    it('throws error if no function is shipped', () => {
      expect(() => {
        both.on('dummyCommand');
      }).to.throw(Error);
    });

    it('throws error if nothing is shipped', () => {
      expect(() => {
        both.on();
      }).to.throw(Error);
    });
  });

  describe('Register listeners', () => {

    it('register listenerFunction as number is no listenerName is given', () => {
      both.on(dummyCommandFunc);

      expect(both._globalRegistery.listener).to.deep.equal({
        0: {
          name: 0,
          func: dummyCommandFunc
        }
      });
    });

    it('register listenerFunction with listenerName if given', () => {
      both.on('dummyCommand', dummyCommandFunc);

      expect(both._globalRegistery.listener).to.deep.equal({
        dummyCommand: {
          name: 'dummyCommand',
          func: dummyCommandFunc
        }
      });
    });
  });
});
