let both    = require('../both');
let sinonChai = require('sinon-chai');
let sinon = require('sinon');
let chai    = require('chai');
chai.use(sinonChai);
let expect  = chai.expect;

describe('Register Module', () => {

  describe('Error handling', () => {

    it('throws an error if no argument is given', () => {
      expect(() => {
        both.register();
      }).to.throw(Error);
    });

    it('throws an error if the first argument isnt a string', () => {
      expect(() => {
        both.register([], {});
      }).to.throw(Error);
    });

    it('throws an error if the second argument isnt a object', () => {
      expect(() => {
        both.register('dummyRegisterType', 'notAllowedSecondArg');
      }).to.throw(Error);
    });

    it('throws an error if no `name` param is in the payload given', () => {
      expect(() => {
        both.register('dummyRegisterType', {
          no: 'nameParamGiven'
        });
      }).to.throw(Error);
    });

  });

  describe('Register Types', () => {

    beforeEach(() => {
      both._globalRegistery = {};
    });

    it('registers correct the registerName in the globalRegistery', () => {
      both._globalRegistery = {};
      both.register('dummyCommands', {
        name: 'dummyCommandName'
      });

      expect(both._globalRegistery).to.deep.equal({
        dummyCommands: {
          dummyCommandName: {
            name: 'dummyCommandName'
          }
        }
      });
    });

    it('registers correct the payload with all object-childrens', () => {
      both._globalRegistery = {};
      both.register('dummyCommands', {
        name: 'dummyCommandName',
        some: 'param'
      });

      expect(both._globalRegistery).to.deep.equal({
        dummyCommands: {
          dummyCommandName: {
            name: 'dummyCommandName',
            some: 'param'
          }
        }
      });
    });

  });

});
