let both    = require('../both');
let sinonChai = require('sinon-chai');
let sinon = require('sinon');
let chai    = require('chai');
chai.use(sinonChai);
let expect  = chai.expect;

describe('Middleware Module', () => {

  describe('Error handling', () => {

    it('throws an error if no argument is given', () => {
      expect(() => {
        both.use();
      }).to.throw(Error);
    });

    it('throws an error if the first argument is no string', () => {
      expect(() => {
        both.use(sinon.spy());
      }).to.throw(Error);
    });

    it('throws an error if no middleware is given', () => {
      expect(() => {
        both.use('dummyMiddlewareName');
      }).to.throw(Error);
    });

    it('throws no error if all arguments are correctly given', () => {
      expect(() => {
        both.use('socket.io', sinon.spy());
      }).to.not.throw(Error);
    });
  });

  describe('Load middleware', () => {

    it('throws no error if module is found', () => {
      expect(() => {
        both.use('socket.io', sinon.spy());
      }).to.not.throw(Error);
    });

    it('throws an error if no module is found', () => {
      expect(() => {
        both.use('dummyMiddlewareName', sinon.spy());
      }).to.throw(Error);
    });

  });
});
